<?php

namespace Drupal\communication\Plugin\Derivative;

use Drupal\communication\CommunicationModePluginManager;
use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

class CommunicationParticipantRoleDeriver extends DeriverBase implements ContainerDeriverInterface {

  /**
   * @var \Drupal\communication\CommunicationModePluginManager
   */
  protected $modeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static($base_plugin_id, $container->get('plugin.manager.communication.mode'));
  }

  /**
   * CommunicationParticipantRoleDeriver constructor.
   *
   * @param $base_plugin_id
   * @param \Drupal\communication\CommunicationModePluginManager $mode_manager
   */
  public function __construct($base_plugin_id, CommunicationModePluginManager $mode_manager) {
    $this->basePluginId = $base_plugin_id;
    $this->modeManager = $mode_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    if (!$this->derivatives) {
      foreach ($this->modeManager->getDefinitions() as $mode => $mode_def) {
        foreach ($this->modeManager->createInstance($mode)->participantRoles() as $role) {
          $derivative_id = $mode.".".$role->getName();
          $plural_name = $role->getPluralName() ?: $role->getName().'s';
          $field_name = $role->getCardinality() === 1 ? $role->getName() : $plural_name;

          $this->derivatives[$derivative_id] = [
            "label" => new TranslatableMarkup('@role Participant', ['@role' => $role->getLabel()]),
            "mode" => $mode,
            "role" => $role->getName(),
            "participant_type" => $role->getType(),
            "entity_type_id" => "communication",
            "bundle" => $mode,
            "field_name" => $field_name,
            "applies_to" => ["entity:communication:{$mode}"]
          ] + $base_plugin_definition;
        }
      }
    }

    return parent::getDerivativeDefinitions($base_plugin_definition); // TODO: Change the autogenerated stub
  }

}
